/********************************************************************************
 * Copyright (c) 2023 CEA-List
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0.
 *
 * SPDX-License-Identifier: EPL-2.0
 *
 ********************************************************************************/

#include <cstddef>
#include <string>
#include <vector>

#include "aidge/operator/Reshape.hpp"
#include "aidge/utils/Types.h"
#include "aidge/utils/ErrorHandling.hpp"

const std::string Aidge::Reshape_Op::Type = "Reshape";

void Aidge::Reshape_Op::computeOutputDims() {
    // check inputs have been associated
    if (!getInput(0)) {
        AIDGE_THROW_OR_ABORT(std::runtime_error, "Input was not connected");
    }

    DimSize_t nbOutDims = this->template getAttr<ReshapeAttr::Shape>().size();
    std::vector<DimSize_t> outDims;
    std::size_t outSize = 1;
    for(std::size_t i=0; i<nbOutDims; ++i)
    {
        int dimSize = this->template getAttr<ReshapeAttr::Shape>()[i];
        if (dimSize < 1)
        {
            AIDGE_THROW_OR_ABORT(std::runtime_error, "bad dimension value");
        }
        outDims.push_back(dimSize);
        outSize *= dimSize;
    }

    if (getInput(0)->size() != outSize){
        AIDGE_THROW_OR_ABORT(std::runtime_error, "Output shape must give the same size as input");
    }

    mOutputs[0]->resize(outDims);
}